@@ -8,6 +8,9 @@ import tempfile
 import struct
 import sys
 import hashlib
+import esptool
+
+from .idrc_esp_tools.gen_esp32part import PartitionTable
 
 
 logger = logging.getLogger('configurer')
@@ -30,94 +33,64 @@ class SerialConfigurator:
 
         nvs_items = nvs_items or []
 
-        partition_table_out = tempfile.NamedTemporaryFile(delete=False, dir=TMP_DIR, suffix='.bin')
-        nvs_partition_out = tempfile.NamedTemporaryFile(delete=False, dir=TMP_DIR, suffix='.bin')
-        nvs_csv_out = tempfile.NamedTemporaryFile(delete=False, dir=TMP_DIR, suffix='.csv')
+        os.makedirs(TMP_DIR, exist_ok=True)
 
-        try:
+        with tempfile.NamedTemporaryFile(dir=TMP_DIR, suffix='.bin') as f:
 
             # dump partition table from a running node
-            p = subprocess.run([
-                'python3', '-m', 'esptool',
-                '--after',
-                'no_reset',
+            esptool.main([
                 '-p',
                 self.usb_port,
+                '--after',
+                'no_reset',
                 'read_flash',
                 '0x8000',
                 '0x1000',
-                partition_table_out.name,
-            ], capture_output=True, text=True)
-            if p.stderr:
-                logger.warning(p.stderr)
-            p.check_returncode()
+                f.name,
+            ])
             logger.info('Dumped partition table')
 
-            # read partition table to determine offset / size
-            nvs_partition_offset = None
-            nvs_partition_size = None
-            p = subprocess.run([
-                'python3',
-                os.path.join(BASE_DIR, 'idrc_esp_tools/gen_esp32part.py'),
-                '-q',
-                partition_table_out.name,
-            ], capture_output=True, text=True)
-            if p.stderr:
-                logger.warning(p.stderr)
-            p.check_returncode()
-            for line in p.stdout.split('\n'):
-                if ',data,nvs,' in line:
-                    _name, _type, _subtype, _offset, _size, _flags = line.strip().split(',')
-                    nvs_partition_offset = int(_offset)
-                    nvs_partition_size = int(_size)
-                    logger.info('Detected NVS partition sized %d', nvs_partition_size)
-            if nvs_partition_offset is None:
-                logger.critical('Cannot find NVS partition!')
-                sys.exit(1)
+            f.seek(0)
+            partition_table = PartitionTable.from_binary(f.read())
+            nvs_partition = partition_table.find_by_name('nvs')
+            assert nvs_partition, 'NVS partition not found!'
+            logger.info('NVS partition @%s len: %s', nvs_partition.offset, nvs_partition.size)
+
+        with tempfile.NamedTemporaryFile(dir=TMP_DIR, mode='wt', suffix='.csv') as csv_out:
 
             # generate CSV
-            with open(nvs_csv_out.name, 'wt') as csv_out:
-                csv_out.write('key,type,encoding,value\n')
-                csv_out.write(f'{NVS_TARGET_NAMESPACE},namespace,,\n')
-                for item in nvs_items:
-                    csv_out.write(f'{item["key"]},data,{item["encoding"]},{item["value"]}\n')
+            csv_out.write('key,type,encoding,value\n')
+            csv_out.write(f'{NVS_TARGET_NAMESPACE},namespace,,\n')
+            for item in nvs_items:
+                csv_out.write(f'{item["key"]},data,{item["encoding"]},{item["value"]}\n')
+            csv_out.flush()
 
-            # generate partition
-            p = subprocess.run([
-                'python3',
-                os.path.join(BASE_DIR, 'idrc_esp_tools/nvs_partition_gen.py'),
-                'generate',
-                nvs_csv_out.name,
-                nvs_partition_out.name,
-                str(nvs_partition_size),
-            ], capture_output=True, text=True)
-            if p.stderr:
-                logger.warning(p.stderr)
-            p.check_returncode()
-            logger.info('Generated new NVS partition')
+            with tempfile.NamedTemporaryFile(dir=TMP_DIR, suffix='.bin') as nvs_partition_out:
 
-            # flash NVS partition
-            p = subprocess.run([
-                'python3', '-m', 'esptool',
-                '-p',
-                self.usb_port,
-                'write_flash',
-                str(nvs_partition_offset),
-                nvs_partition_out.name,
-            ], capture_output=True, text=True)
-            if p.stderr:
-                logger.warning(p.stderr)
-            p.check_returncode()
-            logger.info('Flashed new NVS partition')
+                # generate partition
+                p = subprocess.run([
+                    'python3',
+                    os.path.join(BASE_DIR, 'idrc_esp_tools/nvs_partition_gen.py'),
+                    'generate',
+                    csv_out.name,
+                    nvs_partition_out.name,
+                    str(nvs_partition.size),
+                ], capture_output=True, text=True)
+                if p.stderr:
+                    logger.warning(p.stderr)
+                p.check_returncode()
+                logger.info('Generated new NVS partition')
 
-        finally:
+                # flash NVS partition
+                esptool.main([
+                    '-p',
+                    self.usb_port,
+                    'write_flash',
+                    str(nvs_partition.offset),
+                    nvs_partition_out.name,
+                ])
+                logger.info('Flashed new NVS partition')
 
-            # cleanup
-            os.unlink(nvs_csv_out.name)
-            os.unlink(nvs_partition_out.name)
-            os.unlink(partition_table_out.name)
-
-        # TODO: start testing
         logger.info('Ready!')
 
 
@@ -163,18 +136,19 @@ def main(
     except Exception:
         raise click.BadParameter('mesh_id is invalid')
 
-    ap_sha256 = list(hashlib.sha256(router_password.encode()).digest())
-    mesh_sha256 = list(hashlib.sha256(mesh_password.encode()).digest())
-    encoded_mesh_password = ['0'] * 64
-
-    for i in range(32):
-        mesh_sha256[i] ^= ap_sha256[i]
-        tmp = ((mesh_sha256[i] & 0xf0) >> 4) + ord(b'0')
-        encoded_mesh_password[2 * i] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
-        tmp = (mesh_sha256[i] & 0x0f) + ord(b'0')
-        encoded_mesh_password[2 * i + 1] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
-
-    encoded_mesh_password = ''.join(encoded_mesh_password)[:63]
+    if len(mesh_password) == 63:
+        encoded_mesh_password = mesh_password
+    else:
+        ap_sha256 = list(hashlib.sha256(router_password.encode()).digest())
+        mesh_sha256 = list(hashlib.sha256(mesh_password.encode()).digest())
+        encoded_mesh_password = ['0'] * 64
+        for i in range(32):
+            mesh_sha256[i] ^= ap_sha256[i]
+            tmp = ((mesh_sha256[i] & 0xf0) >> 4) + ord(b'0')
+            encoded_mesh_password[2 * i] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
+            tmp = (mesh_sha256[i] & 0x0f) + ord(b'0')
+            encoded_mesh_password[2 * i + 1] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
+        encoded_mesh_password = ''.join(encoded_mesh_password)[:63]
 
     print(f'''
 port = {port}
