@@ -82,6 +82,46 @@ async def test_shtc3(AT):
         logger.info('SHTC3 humidity offset: %f', test_calibration_humidity - float(humidity))
 
 
+async def test_ds18b20(AT):
+    """Test DS18B20
+
+    This function internally uses the ``TESTDS18B20`` cat command.
+    """
+
+    await AT.TESTDS18B20.execute()
+    temperature = await AT.TESTDS18B20.expect_reply()
+    logger.info('DS18B20 temperature %s', temperature)
+    # FIXME: TODO: RPi could be connected to a calibration source and compare these to some ground truth
+    assert 10 < float(temperature) < 50, 'DS18B20 returned erroneous temperature'
+
+
+async def test_modbus(AT):
+    """Test Modbus
+
+    This function internally uses the ``TESTMODBUS`` cat command.
+    """
+
+    await AT.TESTMODBUS.set(
+        # slave address: raw value
+        1,
+        # baud rate: raw value (ex: 1200 or 2400 or 4800 or 9600 or 14400 or ...)
+        9600,
+        # parity: [1 = even][2 = odd][3 = none] (cf. protocol)
+        3,
+        # function code: [1 = discret input][2 = coil][3 = input register][4 = holding register] (cf protocol)
+        3,
+        # register address: raw value
+        10,
+        # word/byte order: [1 = big endian][2 = little endian] (cf protocol)
+        2,
+        2,
+        # encoding: [1 = bool][2 = float][3 = double][4 = int8][5 = int16][6 = int32][7 = int64][8 = uint8][9 = uint16][10 = uint32][11 = uint64] (cf protocol)
+        9,
+    )
+    result = await AT.TESTMODBUS.expect_reply()
+    assert int(result) == 42, 'Modbus returned erroneous result'
+
+
 async def test_buzzer(AT):
     """Test buzzer
     
